import os
import json
import logging

import requests

from django.conf import settings
from django.utils.translation import ugettext as _
from django.core.exceptions import ImproperlyConfigured

logger = logging.getLogger('mgsub')


class MailgunError(Exception):
    def __init__(self, code, reason=None, json=None):
        self.code = code
        self.reason = reason
        self.json = json

    def __str__(self):
        if self.json is not None:
            return repr(self.code) + ' ' + repr(self.json)
        return repr(self.code) + ' ' + repr(self.reason)


class MailgunList(object):
    list_url = "https://api.mailgun.net/v2/lists/{0}/members"

    def __init__(self, list_email=None):
        if list_email:
            self.list_email = list_email
        elif (hasattr(settings, 'MGSUB_DEFAULT_MAILINGLIST') and
              settings.MGSUB_DEFAULT_MAILINGLIST is not None):
            self.list_email = settings.MGSUB_DEFAULT_MAILINGLIST
        elif 'MGSUB_DEFAULT_MAILINGLIST' in os.environ:
            self.list_email = os.environ.get('MGSUB_DEFAULT_MAILINGLIST')
        else:
            raise ImproperlyConfigured('MGSUB_DEFAULT_MAILINGLIST not set or ' +
                                       'passed directly to MailgunList')

        self.list_email = _(self.list_email)

        if hasattr(settings, 'MAILGUN_API_KEY'):
            self.api_key = settings.MAILGUN_API_KEY
        else:
            self.api_key = os.environ.get('MAILGUN_API_KEY')

    def get_list_url(self):
        return self.list_url.format(self.list_email)

    def get_member_url(self, email):
        return os.path.join(self.get_list_url(), email)

    def _parse_error_response(self, response):
        content = None
        if all([response.headers['content-length'] != '0',
                response.headers['content-type'] == 'application/json']):
            content = response.json()

        return [response.status_code, response.reason, content]

    def subscribe(self, email, name=None, description=None,
                  recipient_vars=None):
        res = requests.post(self.get_list_url(),
                            auth=('api', self.api_key),
                            data={'subscribed': True,
                                  'address': email,
                                  'name': name,
                                  'description': description,
                                  'vars': json.dumps(recipient_vars)},
                            )

        log = {"email": email,
               "list": self.list_email,
               }

        if res.status_code != 200:
            if res.status_code == 400:
                try:
                    jsonres = res.json()

                    # Signing up again is not worth an exception
                    if 'Address already exists' in jsonres['message']:
                        jsonres['member'] = {'address': email}

                        logger.warning(
                            'Duplicate subscription "{email}" to list "{list}".'.format(**log))
                        return jsonres
                except:
                    pass

            logger.info('Error subscribing "{email}" to list "{list}".\n'.format(**log))
            raise MailgunError(*(self._parse_error_response(res)))

        logger.info('Subscribed "{email}" to list "{list}".')
        return res.json()

    def unsubscribe(self, email):
        res = requests.delete(self.get_member_url(email),
                              auth=('api', self.api_key))

        log = {"email": email,
               "list": self.list_email,
               }

        if res.status_code != 200:
            logger.info('Error unsubscribing "{email}" from list "{list}".\n'.format(**log))
            raise MailgunError(*(self._parse_error_response(res)))

        return res.json()



from django.conf.urls import url
from mgsub.views import SignupView, ThanksView

urlpatterns = [
    url(r'^signup/$', SignupView.as_view(), name='signup'),
    url(r'^thanks/$', ThanksView.as_view(), name='thanks'),
]



import logging

import six
from django import forms
from django.conf import settings
from django.template import Context
from django.utils.translation import ugettext as _
from django.core.mail import EmailMultiAlternatives
from django.template.loader import get_template, TemplateDoesNotExist

from mgsub.mailgun import MailgunList

logger = logging.getLogger('mgsub')


class SignupForm(forms.Form):
    email = forms.EmailField()

    def __init__(self, list_email=None, *args, **kwargs):
        self.mailinglist = MailgunList(list_email)
        super(SignupForm, self).__init__(*args, **kwargs)

    def is_valid(self):
        if not super(SignupForm, self).is_valid():
            return False

        if not self.subscribe():
            self.add_error(None, 'There was a failure adding you to the mailing list')
            return False

        if getattr(settings, 'MGSUB_SEND_WELCOME', True):
            return self.send_welcome()

        return True

    def subscribe(self):
        try:
            return self.mailinglist.subscribe(self.cleaned_data['email'])
        except Exception, e:
            logger.error(e)
            return False

    def unsubscribe(self):
        try:
            return self.mailinglist.unsubscribe(self.cleaned_data['email'])
        except Exception, e:
            logger.error(e)
            return False

    def send_welcome(self):
        email = self.cleaned_data['email']

        subject = _(getattr(settings, 'MGSUB_WELCOME_SUBJECT', 'Welcome!'))
        from_address = _(getattr(settings, 'MGSUB_WELCOME_FROM',
                         settings.SERVER_EMAIL))
        reply_to = _(getattr(settings, 'MGSUB_WELCOME_REPLY_TO', None))
        welcome_template = getattr(settings, 'MGSUB_WELCOME_TEMPLATE',
                                   'mgsub/welcome.html')
        welcome_plain = getattr(settings, 'MGSUB_WELCOME_TEMPLATE_PLAIN',
                                'mgsub/welcome.txt')

        if reply_to is not None and isinstance(reply_to, six.string_types):
            reply_to = [reply_to]

        context = Context()

        welcome_txt = get_template(welcome_plain).render(context)

        attach_html = True

        try:
            welcome_html = get_template(welcome_template).render(context)
        except TemplateDoesNotExist:
            attach_html = False

        message = EmailMultiAlternatives(subject, welcome_txt, from_address,
                                         to=[email], reply_to=reply_to)
        if attach_html:
            message.attach_alternative(welcome_html, 'text/html')
        message.send()

        return True



from django.views.generic import TemplateView
from django.views.generic.edit import FormView
from django.core.urlresolvers import reverse_lazy

from mgsub.forms import SignupForm


class SignupView(FormView):
    template_name = 'mgsub/signup.html'
    form_class = SignupForm
    success_url = reverse_lazy('mgsub:thanks')


class ThanksView(TemplateView):
    template_name = 'mgsub/thanks.html'



from mock import patch

import responses
from django.test import TestCase
from django.test.utils import override_settings

from mgsub.mailgun import MailgunList, MailgunError


@override_settings(MGSUB_DEFAULT_MAILINGLIST='list@example.com')
class MailinglistTestCase(TestCase):

    def test_list_url(self):
        mailinglist = MailgunList()
        url = mailinglist.get_list_url()
        exp = 'https://api.mailgun.net/v2/lists/list@example.com/members'
        self.assertEquals(url, exp)

    def test_explicit_list_url(self):
        with patch.dict('os.environ', {'MGSUB_DEFAULT_MAILINGLIST':
                                       'list3@example.com'}, clear=True):
            mailinglist = MailgunList('list2@example.com')
            url = mailinglist.get_list_url()
            exp = 'https://api.mailgun.net/v2/lists/list2@example.com/members'
            self.assertEquals(url, exp)

    @override_settings(MGSUB_DEFAULT_MAILINGLIST=None)
    def test_env_list_url(self):
        with patch.dict('os.environ', {'MGSUB_DEFAULT_MAILINGLIST':
                                       'list3@example.com'}, clear=True):
            mailinglist = MailgunList()
            url = mailinglist.get_list_url()
            exp = 'https://api.mailgun.net/v2/lists/list3@example.com/members'
            self.assertEquals(url, exp)

    def test_api_key(self):
        with patch.dict('os.environ', {}, clear=True):
            mailinglist = MailgunList()
            self.assertIsNone(mailinglist.api_key)

    def test_api_environ(self):
        with patch.dict('os.environ', {'MAILGUN_API_KEY': 'env'}):
            mailinglist = MailgunList()
            self.assertEqual(mailinglist.api_key, 'env')

    @override_settings(MAILGUN_API_KEY='settings')
    def test_api_settings(self):
        mailinglist = MailgunList()
        self.assertEqual(mailinglist.api_key, 'settings')

    def test_member_url(self):
        mailinglist = MailgunList()
        url = mailinglist.get_member_url('sample@example.com')
        self.assertIn('list@example.com/members/sample@example.com', url)

    @responses.activate
    def test_subscribe_success(self):
        resp = \
            '''{
                 "member": {
                   "vars": {
                     "age": 26
                   },
                   "name": "Bob Bar",
                   "subscribed": true,
                   "address": "bar@example.com"
                 },
                 "message": "Mailing list member has been created"
            }'''
        url = 'https://api.mailgun.net/v2/lists/list@example.com/members'
        responses.add(responses.POST, url,
                      resp, content_type='application/json')

        mailinglist = MailgunList()
        result = mailinglist.subscribe('bob@example.com', 'Bob Bar')
        self.assertEquals(result['member']['address'], 'bar@example.com')

    @responses.activate
    def test_unsubscribe_success(self):
        resp = \
            '''{
                 "member":
                   {
                     "address": "bar@example.com"
                   },
                   "message": "Mailing list member has been deleted"
               }'''
        url = 'https://api.mailgun.net/v2/lists/list@example.com/members/bob@example.com'
        responses.add(responses.DELETE, url,
                      resp, content_type='application/json')

        mailinglist = MailgunList()
        result = mailinglist.unsubscribe('bob@example.com')
        self.assertEquals(result['member']['address'], 'bar@example.com')

    @responses.activate
    def test_subscribe_duplicate(self):
        resp = \
            '''{
                 "message": "Address already exists"
               }'''
        url = 'https://api.mailgun.net/v2/lists/list@example.com/members'
        responses.add(responses.POST, url,
                      resp, content_type='application/json',
                      status=400,
                      adding_headers={'content-length': str(len(resp))})

        mailinglist = MailgunList()
        result = mailinglist.subscribe('bob@example.com', 'Bob Bar')
        self.assertEquals(result['member']['address'], 'bob@example.com')

    @responses.activate
    def test_subscribe_400(self):
        resp = \
            '''{
                 "message": "Bad request"
               }'''
        url = 'https://api.mailgun.net/v2/lists/list@example.com/members'
        responses.add(responses.POST, url,
                      resp, content_type='application/json',
                      status=400,
                      adding_headers={'content-length': str(len(resp))})

        mailinglist = MailgunList()
        with self.assertRaises(MailgunError) as cm:
            mailinglist.subscribe('bob@example.com', 'Bob Bar')
        self.assertEqual(400, cm.exception.code)
        self.assertEqual('Bad request', cm.exception.json['message'])

    @responses.activate
    def test_unsubscribe_400(self):
        resp = \
            '''{
                 "message": "Bad request"
               }'''
        url = 'https://api.mailgun.net/v2/lists/list@example.com/members/bob@example.com'
        responses.add(responses.DELETE, url,
                      resp, content_type='application/json',
                      status=400,
                      adding_headers={'content-length': str(len(resp))})

        mailinglist = MailgunList()
        with self.assertRaises(MailgunError) as cm:
            mailinglist.unsubscribe('bob@example.com')

        self.assertEqual(400, cm.exception.code)
        self.assertEqual('Bad request', cm.exception.json['message'])



#!/usr/bin/env python

import os
import versioneer
from pip.download import PipSession
from pip.req import parse_requirements
from setuptools import setup


def get_requirements(filename):
    if not os.path.exists(filename):
        return []

    install_reqs = parse_requirements(filename, session=PipSession())
    return [str(ir.req) for ir in install_reqs]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]
    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

setup(name='django-mgsub',
      version=versioneer.get_version(),
      cmdclass=versioneer.get_cmdclass(),
      description='Subscribe Mailgun mailing lists from Django',
      long_description=README,
      author='Ferrix Hovi',
      author_email='ferrix@codetry.fi',
      install_requires=get_requirements('requirements.txt'),
      tests_require=get_requirements('development.txt'),
      packages=['mgsub'],
      package_data=get_package_data('mgsub'),
      include_package_data=True,
      zip_safe=False,
      url='https://github.com/codetry/mgsub/',
      license='MIT License',
      classifiers=[
          'Environment :: Web Environment',
          'Framework :: Django',
          'Framework :: Django :: 1.8',
          'Intended Audience :: Developers',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'License :: OSI Approved :: MIT License',
          'Topic :: Internet :: WWW/HTTP',
          'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
      ],
      )
